/**
 * Sigma.js WebGL Renderer Node Program
 * =====================================
 *
 * Program rendering nodes using GL_POINTS, but that draws an image on top of
 * the classic colored disc.
 * @module
 */
import { Coordinates, Dimensions, NodeDisplayData } from "../../../types";
import { AbstractNodeProgram, RenderNodeParams } from "./common/node";
declare type ImageLoading = {
    status: "loading";
};
declare type ImageError = {
    status: "error";
};
declare type ImagePending = {
    status: "pending";
    image: HTMLImageElement;
};
declare type ImageReady = {
    status: "ready";
} & Coordinates & Dimensions;
declare type ImageType = ImageLoading | ImageError | ImagePending | ImageReady;
export default class NodeProgramImage extends AbstractNodeProgram {
    textureImage: ImageData;
    images: Record<string, ImageType>;
    hasReceivedImages: boolean;
    pendingImagesFrameID?: number;
    texture: WebGLTexture;
    textureLocation: GLint;
    atlasLocation: WebGLUniformLocation;
    constructor(gl: WebGLRenderingContext);
    bind(): void;
    process(data: NodeDisplayData & {
        image?: string;
    }, hidden: boolean, offset: number): void;
    render(params: RenderNodeParams): void;
    /**
     * Helper to load an image:
     */
    private loadImage;
    /**
     * Helper that takes all pending images and adds them into the texture:
     */
    private finalizePendingImages;
    /**
     * Helper to add an image into the texture (it actually generates a new
     * texture):
     */
    private addToTexture;
}
export {};
