"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../../../utils");
var node_image_vert_glsl_1 = __importDefault(require("../shaders/node.image.vert.glsl.js"));
var node_image_frag_glsl_1 = __importDefault(require("../shaders/node.image.frag.glsl.js"));
var node_1 = require("./common/node");
var POINTS = 1, ATTRIBUTES = 8;
function isPowerOf2(value) {
    return (value & (value - 1)) == 0;
}
var NodeProgramImage = /** @class */ (function (_super) {
    __extends(NodeProgramImage, _super);
    function NodeProgramImage(gl) {
        var _this = _super.call(this, gl, node_image_vert_glsl_1.default, node_image_frag_glsl_1.default, POINTS, ATTRIBUTES) || this;
        _this.images = {};
        _this.hasReceivedImages = false;
        _this.textureImage = new ImageData(1, 1);
        // Attribute Location
        _this.textureLocation = gl.getAttribLocation(_this.program, "a_texture");
        // Uniform Location
        var atlasLocation = gl.getUniformLocation(_this.program, "u_atlas");
        if (atlasLocation === null)
            throw new Error("NodeProgramImage: error while getting atlasLocation");
        _this.atlasLocation = atlasLocation;
        // Initialize WebGL texture:
        _this.texture = gl.createTexture();
        gl.bindTexture(gl.TEXTURE_2D, _this.texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 1, 1, 0, gl.RGBA, gl.UNSIGNED_BYTE, new Uint8Array([0, 0, 0, 0]));
        _this.bind();
        return _this;
    }
    NodeProgramImage.prototype.bind = function () {
        _super.prototype.bind.call(this);
        var gl = this.gl;
        gl.enableVertexAttribArray(this.textureLocation);
        gl.vertexAttribPointer(this.textureLocation, 4, gl.FLOAT, false, this.attributes * Float32Array.BYTES_PER_ELEMENT, 16);
    };
    NodeProgramImage.prototype.process = function (data, hidden, offset) {
        var array = this.array;
        var i = offset * POINTS * ATTRIBUTES;
        var imageSource = data.image;
        var imageState = imageSource && this.images[imageSource];
        if (typeof imageSource === "string" && !imageState)
            this.loadImage(imageSource);
        if (hidden) {
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            // Texture:
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            return;
        }
        array[i++] = data.x;
        array[i++] = data.y;
        array[i++] = data.size;
        array[i++] = (0, utils_1.floatColor)(data.color);
        // Reference texture:
        if (imageState && imageState.status === "ready") {
            var _a = this.textureImage, width = _a.width, height = _a.height;
            array[i++] = imageState.x / width;
            array[i++] = imageState.y / height;
            array[i++] = imageState.width / width;
            array[i++] = imageState.height / height;
        }
        else {
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
        }
    };
    NodeProgramImage.prototype.render = function (params) {
        var gl = this.gl;
        var program = this.program;
        gl.useProgram(program);
        gl.uniform1f(this.ratioLocation, 1 / Math.pow(params.ratio, params.nodesPowRatio));
        gl.uniform1f(this.scaleLocation, params.scalingRatio);
        gl.uniformMatrix3fv(this.matrixLocation, false, params.matrix);
        gl.uniform1i(this.atlasLocation, 0);
        gl.drawArrays(gl.POINTS, 0, this.array.length / ATTRIBUTES);
    };
    /**
     * Helper to load an image:
     */
    NodeProgramImage.prototype.loadImage = function (imageSource) {
        var _this = this;
        if (this.images[imageSource])
            return;
        var image = new Image();
        image.addEventListener("load", function () {
            _this.images[imageSource] = {
                status: "pending",
                image: image,
            };
            if (typeof _this.pendingImagesFrameID !== "number") {
                _this.pendingImagesFrameID = requestAnimationFrame(function () { return _this.finalizePendingImages(); });
            }
        });
        image.addEventListener("error", function () {
            _this.images[imageSource] = { status: "error" };
        });
        this.images[imageSource] = { status: "loading" };
        // Load image:
        image.src = imageSource;
    };
    /**
     * Helper that takes all pending images and adds them into the texture:
     */
    NodeProgramImage.prototype.finalizePendingImages = function () {
        var _this = this;
        this.pendingImagesFrameID = undefined;
        var pendingImages = [];
        for (var id in this.images) {
            var state = this.images[id];
            if (state.status === "pending") {
                pendingImages.push({
                    id: id,
                    image: state.image,
                });
            }
        }
        // Add images to texture:
        var xOffset = this.textureImage.width;
        this.addToTexture(pendingImages.map(function (_a) {
            var image = _a.image;
            return image;
        }));
        // Update images state:
        pendingImages.forEach(function (_a) {
            var id = _a.id, image = _a.image;
            _this.images[id] = {
                status: "ready",
                x: xOffset,
                y: 0,
                width: image.width,
                height: image.height,
            };
            xOffset += image.width;
        });
    };
    /**
     * Helper to add an image into the texture (it actually generates a new
     * texture):
     */
    NodeProgramImage.prototype.addToTexture = function (images) {
        var canvas = document.createElement("canvas");
        var ctx = canvas.getContext("2d");
        canvas.width = images.reduce(function (iter, image) { return iter + image.width; }, this.hasReceivedImages ? this.textureImage.width : 0);
        canvas.height = Math.max.apply(Math, __spreadArray([], __read(images.map(function (image) { return image.height; })), false));
        var xOffset = 0;
        if (this.hasReceivedImages) {
            ctx.putImageData(this.textureImage, 0, 0);
            xOffset = this.textureImage.width;
        }
        images.forEach(function (image) {
            ctx.drawImage(image, xOffset, 0);
            xOffset += image.width;
        });
        this.textureImage = ctx.getImageData(0, 0, canvas.width, canvas.height);
        var gl = this.gl;
        gl.bindTexture(gl.TEXTURE_2D, this.texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, this.textureImage);
        if (isPowerOf2(this.textureImage.width) && isPowerOf2(this.textureImage.height)) {
            gl.generateMipmap(gl.TEXTURE_2D);
        }
        else {
            gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
            gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        }
        this.hasReceivedImages = true;
    };
    return NodeProgramImage;
}(node_1.AbstractNodeProgram));
exports.default = NodeProgramImage;
